"""Implements a parent client to interact with a SqLite DB."""
from fvidal_packages.db_api_client.parent_cli import DBAPIClient


class SqLiteClient(DBAPIClient):
    """Children class to connect to a SqLite DB using SQLAlchemy.

    Attributes
    ----------
        db: str
            DB to connect.
    """
    def __init__(self, db):
        """Constructor method for this children class.

        Parameters
        ----------
            db: str
                DB to connect.
        """
        super().__init__('sqlite', db)



"""Implements a parent client to interact with any DB (SqLite or Postgres)."""
import pandas as pd
from sqlalchemy import create_engine


class DBAPIClient:
    """Parent class to connect to a DB using SQLAlchemy.

    Attributes
    ----------
        dialect: str
            dialect to use in the SQLAlchemy engine.
        db: str
            DB to connect.
    """
    def __init__(self, dialect, db):
        """Constructor method for this class.

        Parameters
        ----------
            dialect: str
                dialect to use in the SQLAlchemy engine.
            db: str
                DB to connect.
        """
        self.dialect = dialect
        self.db = db
        self._engine = None

    def _get_engine(self):
        """Creates engine to connect to DB.

        Returns
        -------
            Engine: object
                manages DB connections.
        """
        db_uri = f'{self.dialect}:{self.db}'
        if not self._engine:
            self._engine = create_engine(db_uri)
        return self._engine

    def _connect(self):
        """Connects to a DB.

        Returns
        -------
            Connection: object
                proxy object for DB connections.
        """
        return self._get_engine().connect()

    @staticmethod
    def _cursor_columns(cursor):
        """Gets cursor columns.

        Parameters
        ----------
            cursor: object
                DB cursor.
        Returns
        -------
            list:
                list of columns.
        """
        if hasattr(cursor, 'keys'):
            return cursor.keys()
        else:
            return [c[0] for c in cursor.description]

    def execute(self, sql, connection=None):
        """Executes SQL query.

        Parameters
        ----------
            sql: str
                str representing a query.
            connection: object
                proxy for DB connection.
        Returns
        -------
            ResultProxy: object
                DB cursor object to provide access to row columns.
        """
        if connection is None:
            connection = self._connect()
        return connection.execute(sql)

    def insert_from_frame(self, df, table, if_exists='append', index=False, **kwargs):
        """Inserts a pandas DataFrame into the DB.

        Parameters
        ----------
            df: pandas DataFrame
                DataFrame to be inserted into the DB.
            table: str
                table where DataFrame is inserted.
            if_exists: {"append", "fails", "replace"}
                what to do in case that the table already exists in DB.
            index: bool
                whether to consider the DataFrame index as a column or not.
        """
        connection = self._connect()
        with connection:
            df.to_sql(table, connection, if_exists=if_exists, index=index, **kwargs)

    def to_frame(self, *args, **kwargs):
        """Executes a SQL query and returns a DataFrame.

        Returns
        -------
            df: pandas.DataFrame
                result of the query in a pandas DataFrame.
        """
        cursor = self.execute(*args, **kwargs)
        if not cursor:
            return
        data = cursor.fetchall()
        if data:
            df = pd.DataFrame(data, columns=self._cursor_columns(cursor))
        else:
            df = pd.DataFrame()
        return df



"""Implements a parent client to interact with a Postgres DB."""
from fvidal_packages.db_api_client.parent_cli import DBAPIClient


class PostgresClient(DBAPIClient):
    """Children class to connect to a Postgres DB using SQLAlchemy.

    Attributes
    ----------
        host: str
            dns name of the host of the postgres database.
        port: int
            port where the postgres database is listening.
        user: str
            username to connect to the postgres database.
        password: str
            password for this username.
        database: str
            database to connect
    """
    def __init__(self, user, password, host, port, database):
        """Constructor method for this children class.

        Parameters
        ----------
        host: str
            dns name of the host of the postgres database.
        port: int
            port where the postgres database is listening.
        user: str
            username to connect to the postgres database.
        password: str
            password for this username.
        database: str
            database to connect
        """
        db = f'//{user}:{password}@{host}:{port}/{database}'
        super().__init__('postgresql', db)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fvidal_packages',
 'fvidal_packages.db_api_client',
 'fvidal_packages.stock_utils']

package_data = \
{'': ['*']}

install_requires = \
['SQLAlchemy==1.4.27',
 'kaleido==0.2.1',
 'pandas==1.3.4',
 'plotly>=5.4.0,<6.0.0',
 'psycopg2-binary==2.9.2',
 'requests==2.26.0']

setup_kwargs = {
    'name': 'itba-cde-tpf-python-applications-fvidal90',
    'version': '1.0.1',
    'description': 'TP Vidal',
    'long_description': None,
    'author': 'Fernando Vidal',
    'author_email': 'picovidal22@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
